.hljs{background:#f8f8f8;color:#333;display:block;overflow-x:auto;padding:.5em}.hljs-comment,.hljs-quote{color:#998;font-style:italic}.hljs-keyword,.hljs-selector-tag,.hljs-subst{color:#333;font-weight:700}.hljs-literal,.hljs-number,.hljs-tag .hljs-attr,.hljs-template-variable,.hljs-variable{color:teal}.hljs-doctag,.hljs-string{color:#d14}.hljs-section,.hljs-selector-id,.hljs-title{color:#900;font-weight:700}.hljs-subst{font-weight:400}.hljs-class .hljs-title,.hljs-type{color:#458;font-weight:700}.hljs-attribute,.hljs-name,.hljs-tag{color:navy;font-weight:400}.hljs-link,.hljs-regexp{color:#009926}.hljs-bullet,.hljs-symbol{color:#990073}.hljs-built_in,.hljs-builtin-name{color:#0086b3}.hljs-meta{color:#999;font-weight:700}.hljs-deletion{background:#fdd}.hljs-addition{background:#dfd}.hljs-emphasis{font-style:italic}.hljs-strong{font-weight:700}.highlightjs pre{-moz-border-radius:0;-webkit-border-radius:0;border-color:transparent;border-radius:0}.highlightjs pre,.hljs{background-color:transparent}.hljs{white-space:pre}.hljs-ln-numbers{-moz-user-select:none;-ms-user-select:none;-webkit-user-select:none;border-right:1px solid #ccc;color:#aaa;padding-right:5px!important;text-align:center;user-select:none;vertical-align:top}.hljs-ln-code{padding-left:10px!important;width:100%}