# Copyright (C) 2017-2018  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU Affero General Public License version 3, or any later version
# See top-level LICENSE file for more information

# flake8: noqa

revision_id_test = '7bc08e1aa0b08cb23e18715a32aa38517ad34672'

revision_metadata_test = \
{'id': '7bc08e1aa0b08cb23e18715a32aa38517ad34672',
 'type': 'git',
 'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
             'a952bb99a6830804d06c5b8e04b75c66100fbae9'],
 'metadata': {},
 'committer': {'name': 'GitHub',
               'fullname': 'GitHub <noreply@github.com>',
               'id': 10932771,
               'email': 'noreply@github.com'},
 'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
 'date': '2017-05-04T13:27:13+02:00',
 'merge': True,
 'committer_date': '2017-05-04T13:27:13+02:00',
 'author': {'name': 'Tobias Koppers',
            'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
            'id': 141959,
            'email': 'tobias.koppers@googlemail.com'},
 'message': 'Merge pull request #4816 from webpack/bugfix/hoist-immutable-export\n\nhoist exports',
 'synthetic': False
}

revision_history_log_test = \
[{'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T13:27:13+02:00',
  'date': '2017-05-04T13:27:13+02:00',
  'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
  'id': '7bc08e1aa0b08cb23e18715a32aa38517ad34672',
  'merge': True,
  'message': 'Merge pull request #4816 from '
             'webpack/bugfix/hoist-immutable-export\n'
             '\n'
             'hoist exports',
  'metadata': {},
  'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
              'a952bb99a6830804d06c5b8e04b75c66100fbae9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T12:17:13+02:00',
  'date': '2017-05-04T12:17:13+02:00',
  'directory': '344f6b38f021fa0bbd60ca06fe1cbc61164e7abe',
  'id': 'bf3652b16b65c27db5243aa0d674e2de4a8ccde9',
  'merge': False,
  'message': '2.5.0\n',
  'metadata': {},
  'parents': ['cd1cd29fba46bd0133db0ca89acbe6c6c0240323'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T13:00:52+02:00',
  'date': '2017-05-04T12:56:31+02:00',
  'directory': 'ae59ceecf46367e8e4ad800e231fc76adc3afffb',
  'id': 'a952bb99a6830804d06c5b8e04b75c66100fbae9',
  'merge': False,
  'message': 'change some magic numbers to hoist exports\n\nfixes #4753\n',
  'metadata': {},
  'parents': ['bf3652b16b65c27db5243aa0d674e2de4a8ccde9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T11:41:43+02:00',
  'date': '2017-05-04T11:41:43+02:00',
  'directory': 'fbc01dab452f80bf49d554cc920979a66839707f',
  'id': 'cd1cd29fba46bd0133db0ca89acbe6c6c0240323',
  'merge': True,
  'message': 'Merge pull request #4815 from '
             'webpack/bugfix/extract-async-initial\n'
             '\n'
             "CommonsChunkPlugin in async mode doesn't select initial chunks",
  'metadata': {},
  'parents': ['8bab88c50fcb87c749c244b9ab28a1fb7e173bea',
              'b45588bc1197abbc309eb3705a4bf89960b001ae'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T10:50:03+02:00',
  'date': '2017-05-04T10:50:03+02:00',
  'directory': '74ed598c6b831fe7f68697f6ed761660a842b973',
  'id': '8bab88c50fcb87c749c244b9ab28a1fb7e173bea',
  'merge': True,
  'message': 'Merge pull request #4814 from webpack/test/move-entry\n'
             '\n'
             'add testcase for moving entry modules into the commons chunk',
  'metadata': {},
  'parents': ['85dc98f17aa39d5d3337e3791bf25634a1f7e445',
              'a244879a07e04e6b5951520ca3cd80c3ef160f8e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T10:50:46+02:00',
  'date': '2017-05-04T10:50:46+02:00',
  'directory': '735b9f6d54d62470944c4743d52931b308a9072c',
  'id': 'b45588bc1197abbc309eb3705a4bf89960b001ae',
  'merge': False,
  'message': "CommonsChunkPlugin in async mode doesn't select initial chunks\n"
             '\n'
             'fixes #4795\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-05-04T08:04:41+02:00',
  'date': '2017-05-04T08:04:41+02:00',
  'directory': 'c3c24aeafabe1d441c37a2769e9b65ae10075925',
  'id': '85dc98f17aa39d5d3337e3791bf25634a1f7e445',
  'merge': True,
  'message': 'Merge pull request #4813 from JLHwung/perf/date-now\n'
             '\n'
             'Perf/use Date.now() instead of +new Date()/new Date().getTime()',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb',
              '6afc397b99aef338a9d66add4488ce03ea3f7a43'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-05-04T10:01:04+02:00',
  'date': '2017-05-04T10:01:04+02:00',
  'directory': '4de09be1628ed81def03f78d8d832c93efdf0af4',
  'id': 'a244879a07e04e6b5951520ca3cd80c3ef160f8e',
  'merge': False,
  'message': 'add testcase for moving entry modules into the commons chunk\n'
             '#4795\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-30T09:31:17+02:00',
  'date': '2017-04-30T09:31:17+02:00',
  'directory': '16c9d449871efccef7a0f53b29bd6218cc769e30',
  'id': 'c91ba4949503de5ca9d98c98188c2654b095f2cb',
  'merge': True,
  'message': 'Merge pull request #4791 from deificx/master\n'
             '\n'
             'add option to lib/Stats.js to disable stack trace on errors and '
             'warnings',
  'metadata': {},
  'parents': ['94ba75f7940836390c041846f2c334929ee14332',
              '84ea1ffd3d0892f3356ac0494947bbc2a0e39d51'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'i@jhuang.me',
             'fullname': 'J Huang <i@jhuang.me>',
             'id': 12072398,
             'name': 'J Huang'},
  'committer': {'email': 'i@jhuang.me',
                'fullname': 'J Huang <i@jhuang.me>',
                'id': 12072398,
                'name': 'J Huang'},
  'committer_date': '2017-05-04T10:19:54+08:00',
  'date': '2017-05-04T10:19:54+08:00',
  'directory': 'c3c24aeafabe1d441c37a2769e9b65ae10075925',
  'id': '6afc397b99aef338a9d66add4488ce03ea3f7a43',
  'merge': False,
  'message': 'perf: use Date.now() instead of new Date().getTime()\n'
             '\n'
             'new Date().getTime() is 2x slower than Date.now(), see '
             'https://jsperf.com/new-date-vs-date-now-vs-performance-now/6\n',
  'metadata': {},
  'parents': ['94d0641ba40d65d5fcbd64414b6ea9f8a2589538'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-30T09:30:03+02:00',
  'date': '2017-04-30T09:30:03+02:00',
  'directory': 'f78fe6939706d761f0bc60f9d566bd414ef5c113',
  'id': '94ba75f7940836390c041846f2c334929ee14332',
  'merge': True,
  'message': 'Merge pull request #4794 from '
             'ndresx/disable-manifest-json-pretty-print\n'
             '\n'
             'Disable manifest.json pretty print',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a',
              'de87f93c1b050db59ffbeff3aed4ac3b3eb57da3'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T20:53:42+02:00',
  'date': '2017-04-29T20:52:43+02:00',
  'directory': '68fcb2d3c60aa32b708ce9ca9fa5c4b1f7a2532d',
  'id': '84ea1ffd3d0892f3356ac0494947bbc2a0e39d51',
  'merge': False,
  'message': 'added error to stats.moduleTrace test name to trigger test cases '
             'corretly\n',
  'metadata': {},
  'parents': ['8ad4386bdf9cd607310a4255dff8cb9ccd12afad'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'i@jhuang.me',
             'fullname': 'J Huang <i@jhuang.me>',
             'id': 12072398,
             'name': 'J Huang'},
  'committer': {'email': 'i@jhuang.me',
                'fullname': 'J Huang <i@jhuang.me>',
                'id': 12072398,
                'name': 'J Huang'},
  'committer_date': '2017-05-04T10:19:10+08:00',
  'date': '2017-05-04T10:17:23+08:00',
  'directory': '41965bffa3f73c8928beb2a6580c253adb9bd5cf',
  'id': '94d0641ba40d65d5fcbd64414b6ea9f8a2589538',
  'merge': False,
  'message': 'perf: use Date.now() instead of +new Date()\n'
             '\n'
             '+new Date() is 2x slower than Date.now(), see '
             'https://jsperf.com/new-date-vs-date-now-vs-performance-now/6\n',
  'metadata': {},
  'parents': ['c91ba4949503de5ca9d98c98188c2654b095f2cb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-26T09:05:56+02:00',
  'date': '2017-04-26T09:05:56+02:00',
  'directory': 'aa78629e4aefc836b5a967db57cac29f79fea074',
  'id': '24ef6ea1b56b358caeb4b07476a909f4f86c2d8a',
  'merge': True,
  'message': 'Merge pull request #4779 from ndresx/ignoreplugin-typo-fix\n'
             '\n'
             'Fix typo in IgnorePlugin',
  'metadata': {},
  'parents': ['34315db3ffe20b3d7e9a50885ea623fc90ac0861',
              'aa172f0a359ab3c0b0987101c64c8b41b07d581f'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'ndresx@gmail.com',
             'fullname': 'Martin Veith <ndresx@gmail.com>',
             'id': 14855385,
             'name': 'Martin Veith'},
  'committer': {'email': 'ndresx@gmail.com',
                'fullname': 'Martin Veith <ndresx@gmail.com>',
                'id': 14855385,
                'name': 'Martin Veith'},
  'committer_date': '2017-04-28T21:34:38+02:00',
  'date': '2017-04-28T21:34:38+02:00',
  'directory': 'f78fe6939706d761f0bc60f9d566bd414ef5c113',
  'id': 'de87f93c1b050db59ffbeff3aed4ac3b3eb57da3',
  'merge': False,
  'message': 'Disable manifest.json pretty print\n',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T19:43:45+02:00',
  'date': '2017-04-29T19:43:45+02:00',
  'directory': '893a65a9ad785d719c44a3e764d150bb47cba6c1',
  'id': '8ad4386bdf9cd607310a4255dff8cb9ccd12afad',
  'merge': False,
  'message': 'test cases for stats.moduleTrace option\n',
  'metadata': {},
  'parents': ['958156ae4201e75f8cb04e306d2a38b94d2a8a1d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T14:25:42+02:00',
  'date': '2017-04-23T14:25:42+02:00',
  'directory': 'f511a1febf3d6b9b788080f3eb69ff3cb963758f',
  'id': '34315db3ffe20b3d7e9a50885ea623fc90ac0861',
  'merge': True,
  'message': 'Merge pull request #3875 from webpack/test/circleci\n'
             '\n'
             'Update circle.yml',
  'metadata': {},
  'parents': ['6a26e9ba7f7f1be7e76054f219bf1e094f2c3264',
              '0f91f949e2d41ef5cb92493bcc6c1fa7578ac27d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'ndresx@gmail.com',
             'fullname': 'Martin Veith <ndresx@gmail.com>',
             'id': 14855385,
             'name': 'Martin Veith'},
  'committer': {'email': 'ndresx@gmail.com',
                'fullname': 'Martin Veith <ndresx@gmail.com>',
                'id': 14855385,
                'name': 'Martin Veith'},
  'committer_date': '2017-04-25T22:07:43+02:00',
  'date': '2017-04-25T22:07:43+02:00',
  'directory': 'aa78629e4aefc836b5a967db57cac29f79fea074',
  'id': 'aa172f0a359ab3c0b0987101c64c8b41b07d581f',
  'merge': False,
  'message': 'Fix typo in IgnorePlugin\n',
  'metadata': {},
  'parents': ['34315db3ffe20b3d7e9a50885ea623fc90ac0861'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-29T19:43:15+02:00',
  'date': '2017-04-29T19:43:15+02:00',
  'directory': '9d7347c42e29afda2849996ca6b61cdc7ea1b5a1',
  'id': '958156ae4201e75f8cb04e306d2a38b94d2a8a1d',
  'merge': False,
  'message': 'moduleTrace added to webpackOptionsSchema.json\n',
  'metadata': {},
  'parents': ['41310135bb4e30a0a6f71eddd3d74419ec8512a7'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:57:02+02:00',
  'date': '2017-04-23T12:57:02+02:00',
  'directory': '71848e66952829c98ec4e57e2338be986390b850',
  'id': '6a26e9ba7f7f1be7e76054f219bf1e094f2c3264',
  'merge': True,
  'message': 'Merge pull request #4693 from '
             'Travmatth/fix-4252-BannerPlugin-placeholder\n'
             '\n'
             'Fix 4252 banner plugin placeholder',
  'metadata': {},
  'parents': ['53bb15b1ed64f8636036f773100d502909bd1e6b',
              '08eca2fb4517bb2a14e21fa46e3291878c79be0e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'tobias.koppers@googlemail.com',
                'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
                'id': 141959,
                'name': 'Tobias Koppers'},
  'committer_date': '2017-04-23T14:02:06+02:00',
  'date': '2017-04-23T12:50:01+02:00',
  'directory': '29cf488a00126c0e45c78b92f97cdbc733f4d07e',
  'id': '0f91f949e2d41ef5cb92493bcc6c1fa7578ac27d',
  'merge': False,
  'message': 'improve circleci build\n',
  'metadata': {},
  'parents': ['53bb15b1ed64f8636036f773100d502909bd1e6b'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-28T09:20:14+02:00',
  'date': '2017-04-28T09:20:14+02:00',
  'directory': 'f8b679bd62a33880d7f39f70233c3c14aead7c22',
  'id': '41310135bb4e30a0a6f71eddd3d74419ec8512a7',
  'merge': False,
  'message': 'rename stats.stackTrace to stats.moduleTrace\n',
  'metadata': {},
  'parents': ['7e4310a1759bd35834290e4c4af207ce9b05ffd9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:10:20+02:00',
  'date': '2017-04-23T12:10:20+02:00',
  'directory': '1887ba14662a7e45567fa6daa39ab72975b429aa',
  'id': '53bb15b1ed64f8636036f773100d502909bd1e6b',
  'merge': True,
  'message': 'Merge pull request #3934 from '
             'timse/refactor-watching-in-compiler\n'
             '\n'
             'Refactor _done of Watching in compiler',
  'metadata': {},
  'parents': ['b67d61abd6dd878f7a9aec4d10689c8b1678e28a',
              'ab30c6b1c0100634ad46bcaf47ff12963e22aac9'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:00:53+02:00',
  'date': '2017-04-23T12:00:53+02:00',
  'directory': '4da5fda41a4b5cdbcf1fbb2604e9696c67106271',
  'id': '08eca2fb4517bb2a14e21fa46e3291878c79be0e',
  'merge': False,
  'message': 'spacing',
  'metadata': {},
  'parents': ['7061c2c76b984e9c7ab893bf4b14b4b102b5fa0d'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'even.alander@videxio.com',
             'fullname': 'Even Alander <even.alander@videxio.com>',
             'id': 3407016,
             'name': 'Even Alander'},
  'committer': {'email': 'even.alander@videxio.com',
                'fullname': 'Even Alander <even.alander@videxio.com>',
                'id': 3407016,
                'name': 'Even Alander'},
  'committer_date': '2017-04-27T10:22:20+02:00',
  'date': '2017-04-27T10:22:20+02:00',
  'directory': '21f3f99258fad8513cbdd40f549cb067362add27',
  'id': '7e4310a1759bd35834290e4c4af207ce9b05ffd9',
  'merge': False,
  'message': 'add option to lib/Stats.js to disable stack trace on errors and '
             'warnings\n',
  'metadata': {},
  'parents': ['24ef6ea1b56b358caeb4b07476a909f4f86c2d8a'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:04:28+02:00',
  'date': '2017-04-23T12:04:28+02:00',
  'directory': 'effe11da8d44198a760cab867351f66d4b0da7d1',
  'id': 'b67d61abd6dd878f7a9aec4d10689c8b1678e28a',
  'merge': True,
  'message': 'Merge pull request #4755 from aretecode/lint-update\n'
             '\n'
             'update node linting',
  'metadata': {},
  'parents': ['a2ec4c8cea281715d3043a6dcb4198be039cfb54',
              '5c423d6feb6521c9b5274ba342b7621c453f35eb'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-08T09:13:38+10:00',
  'date': '2017-04-08T09:13:38+10:00',
  'directory': '20805e5ff5776062b9108138c4dfce8124185f5a',
  'id': 'ab30c6b1c0100634ad46bcaf47ff12963e22aac9',
  'merge': False,
  'message': 'pass stats to error handler again, to prevent breaking change\n',
  'metadata': {},
  'parents': ['b863851ce969317688799b754131e3546206c7ad'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T09:46:34-04:00',
  'date': '2017-04-19T09:46:34-04:00',
  'directory': '3c9ea2ac7fec857634b7b46dc08b52a5670106c5',
  'id': '7061c2c76b984e9c7ab893bf4b14b4b102b5fa0d',
  'merge': False,
  'message': 'added file test, remove unused [basename]\n',
  'metadata': {},
  'parents': ['527f9434f74e63ae76ecbf18a4710589667dc731'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T12:02:55+02:00',
  'date': '2017-04-23T12:02:55+02:00',
  'directory': '5185e53216f3a836598f946f7302155f5bb50d02',
  'id': 'a2ec4c8cea281715d3043a6dcb4198be039cfb54',
  'merge': True,
  'message': 'Merge pull request #4722 from '
             'willmendesneto/refactor-format-location\n'
             '\n'
             'refactor(formatLocation): upgrade to ES6',
  'metadata': {},
  'parents': ['1c5f3bf59f8a1e06bbe83b7459362f69bb5b7460',
              'c826edde1be8af20a02166f792f79bbf2a231c4e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-22T19:02:01+02:00',
  'date': '2017-04-22T19:02:01+02:00',
  'directory': '461c61e52405649d272d7e79b8a1abe27e5b33b2',
  'id': '5c423d6feb6521c9b5274ba342b7621c453f35eb',
  'merge': False,
  'message': 'change supported node.js version to 4',
  'metadata': {},
  'parents': ['219a95dddca951737d4b947fc2d59e3a2b5cbb23'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-06T21:52:06+10:00',
  'date': '2017-04-06T21:52:06+10:00',
  'directory': '803b83492f4be6b472720dfd5dd0f45675213ac2',
  'id': 'b863851ce969317688799b754131e3546206c7ad',
  'merge': False,
  'message': 'add getStats again - remove breaking change\n',
  'metadata': {},
  'parents': ['2a1bcff9e6999d471747ab3575d82c33a0e5e082'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T09:28:25-04:00',
  'date': '2017-04-19T09:28:25-04:00',
  'directory': 'aafcbc664c51bf17dcfde6b7112efe95331c6fcd',
  'id': '527f9434f74e63ae76ecbf18a4710589667dc731',
  'merge': False,
  'message': 'removed single quotes\n',
  'metadata': {},
  'parents': ['c68bd16b265b6d5f8478e5728ea077ead90e8a37'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-23T11:49:28+02:00',
  'date': '2017-04-23T11:49:28+02:00',
  'directory': 'cf61c993ed0335a88d7975aba07ef743eaa7b845',
  'id': '1c5f3bf59f8a1e06bbe83b7459362f69bb5b7460',
  'merge': True,
  'message': 'Merge pull request #4748 from '
             'aretecode/examples-dll-readme-update2\n'
             '\n'
             'examples/dll readme updates',
  'metadata': {},
  'parents': ['cbd493904f222abac1d33918b4686b0bc4de3a42',
              'f223aecd05b47b9cfff34609f730c81d03034514'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'willmendesneto@gmail.com',
             'fullname': 'Will Mendes <willmendesneto@gmail.com>',
             'id': 93385,
             'name': 'Will Mendes'},
  'committer': {'email': 'willmendesneto@gmail.com',
                'fullname': 'Will Mendes <willmendesneto@gmail.com>',
                'id': 93385,
                'name': 'Will Mendes'},
  'committer_date': '2017-04-17T22:00:47+10:00',
  'date': '2017-04-17T22:00:47+10:00',
  'directory': '817b5de0d36aae8f3243b472e19b2fae1adf9173',
  'id': 'c826edde1be8af20a02166f792f79bbf2a231c4e',
  'merge': False,
  'message': 'refactor(formatLocation): upgrade to ES6\n',
  'metadata': {},
  'parents': ['82ddd16080663e3773bdac2a62082991acb8637e'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'James <aretecode@gmail.com>',
             'id': 11020868,
             'name': 'James'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-19T23:53:44-07:00',
  'date': '2017-04-19T23:53:44-07:00',
  'directory': 'fa17b78edd5fd19a589663489e91b14e2c6337c0',
  'id': '219a95dddca951737d4b947fc2d59e3a2b5cbb23',
  'merge': False,
  'message': 'lint autofix the eslint file',
  'metadata': {},
  'parents': ['cf666188298dfe75cc343018bf35b1c753fb6636'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tim.sebastian@gmail.com',
             'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
             'id': 3654888,
             'name': 'Tim Sebastian'},
  'committer': {'email': 'tim.sebastian@gmail.com',
                'fullname': 'Tim Sebastian <tim.sebastian@gmail.com>',
                'id': 3654888,
                'name': 'Tim Sebastian'},
  'committer_date': '2017-04-06T21:46:07+10:00',
  'date': '2017-01-12T22:33:03+11:00',
  'directory': '7c6b85c086b56f11cf29bc3d13836145dc225407',
  'id': '2a1bcff9e6999d471747ab3575d82c33a0e5e082',
  'merge': False,
  'message': 'refactor the _done method of Watching\n'
             '\n'
             '- we can safely ignore the "else" cases of not having an error '
             'as _done() is only called without arguments if "this.invalid is '
             'true"\n'
             '- if we get passed the point of `this.invalid` either `err` or '
             '`compilation` are !!always!! set. therefore later checks can '
             'again be ignored\n'
             '- early return in error case\n'
             '- ignore `this.error` if we make it passed the error as it will '
             'be unset at this point.\n'
             '- remove the setting of `this.error` or `this.stats` as the only '
             'use is inside this method and only allow weird behaviour if '
             'someone set them from outside\n',
  'metadata': {},
  'parents': ['8165164d3e50fef3252d7c274cef1c0b595c6992'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'trav.matth@gmail.com',
             'fullname': 'Travis Matthews <trav.matth@gmail.com>',
             'id': 439443,
             'name': 'Travis Matthews'},
  'committer': {'email': 'trav.matth@gmail.com',
                'fullname': 'Travis Matthews <trav.matth@gmail.com>',
                'id': 439443,
                'name': 'Travis Matthews'},
  'committer_date': '2017-04-19T08:41:00-04:00',
  'date': '2017-04-19T08:41:00-04:00',
  'directory': '34bfecaa341626e1c60d1d2a1f697fb1f74f8bcd',
  'id': 'c68bd16b265b6d5f8478e5728ea077ead90e8a37',
  'merge': False,
  'message': 'remove comment\n',
  'metadata': {},
  'parents': ['f2e30693ac0e3a9a0fe6a01a89fe6072dee7f405'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-22T12:45:04+02:00',
  'date': '2017-04-22T12:45:04+02:00',
  'directory': '412c6f5c00ce7d02efcdd6f23160a36eb4e5a1f4',
  'id': 'cbd493904f222abac1d33918b4686b0bc4de3a42',
  'merge': True,
  'message': 'Merge pull request #4768 from xizhao/patch-1\n'
             '\n'
             'Add license scan report and status',
  'metadata': {},
  'parents': ['d7f30392ddda27613e0ea05cb60ec985b4f75e5c',
              'ff211108d888908d41470cea6187133ccdb56e87'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'Arete Code <aretecode@gmail.com>',
             'id': 8227899,
             'name': 'Arete Code'},
  'committer': {'email': 'aretecode@gmail.com',
                'fullname': 'Arete Code <aretecode@gmail.com>',
                'id': 8227899,
                'name': 'Arete Code'},
  'committer_date': '2017-04-21T15:48:06-07:00',
  'date': '2017-04-21T15:48:06-07:00',
  'directory': '9d69f66cf3d2fa5119910bfd6a16f16e996795cc',
  'id': 'f223aecd05b47b9cfff34609f730c81d03034514',
  'merge': False,
  'message': 'rebase for semicolon\n',
  'metadata': {},
  'parents': ['914fe2c923533d43f5edff54b1704bbd3dd407bc'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'tobias.koppers@googlemail.com',
             'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
             'id': 141959,
             'name': 'Tobias Koppers'},
  'committer': {'email': 'noreply@github.com',
                'fullname': 'GitHub <noreply@github.com>',
                'id': 10932771,
                'name': 'GitHub'},
  'committer_date': '2017-04-15T19:26:54+02:00',
  'date': '2017-04-15T19:26:54+02:00',
  'directory': 'c2da553b359190d3dc4266d77e311bd002d7ccd2',
  'id': '82ddd16080663e3773bdac2a62082991acb8637e',
  'merge': True,
  'message': 'Merge pull request #4717 from STRML/fix/hashSalt\n'
             '\n'
             'fix missing `hashSalt` from options schema',
  'metadata': {},
  'parents': ['bd753567da1248624beaaea14af31d6dbe303411',
              '805c9fadf05c00f3df16d1faf4d3f608b78e3b59'],
  'synthetic': False,
  'type': 'git'},
 {'author': {'email': 'aretecode@gmail.com',
             'fullname': 'Arete Code <aretecode@gmail.com>',
             'id': 8227899,
             'name': 'Arete Code'},
  'committer': {'email': 'aretecode@gmail.com',
                'fullname': 'Arete Code <aretecode@gmail.com>',
                'id': 8227899,
                'name': 'Arete Code'},
  'committer_date': '2017-04-19T19:05:28-07:00',
  'date': '2017-04-19T19:05:28-07:00',
  'directory': 'dd7dc17a8a28b6bb415b1f6980de6fcd3715847c',
  'id': 'cf666188298dfe75cc343018bf35b1c753fb6636',
  'merge': False,
  'message': '👕 update node linting\n'
             '\n'
             'update linting for destructuring and latest node\n'
             '\n'
             'uses .js rather than .eslintrc file\n'
             '\n'
             'pulls in rules from\n'
             'https://github.com/webpack/webpack-cli/pull/46/files#diff-df39304d828831c44a2b9f38cd45289cR40\n'
             '\n'
             'adds spacing for this\n'
             '<img width="495" alt="screen shot 2017-04-19 at 7 03 16 pm" '
             'src="https://cloud.githubusercontent.com/assets/4022631/25210014/eabff164-2532-11e7-89a3-837fde6c7101.png">\n',
  'metadata': {},
  'parents': ['8aa8a7b63fed3c7909a6d4f15159c036a0561d64'],
  'synthetic': False,
  'type': 'git'},
 {'author': {
    'email': 'tobias.koppers@googlemail.com',
    'fullname': 'Tobias Koppers <tobias.koppers@googlemail.com>',
    'id': 141959,
    'name': 'Tobias Koppers'
  },
  'committer': {
    'email': 'noreply@github.com',
    'fullname': 'GitHub <noreply@github.com>',
    'id': 10932771,
    'name': 'GitHub'
  },
  'committer_date': '2017-04-21T19:00:50+02:00',
  'committer_url': '/api/1/person/10932771/',
  'date': '2017-04-21T19:00:50+02:00',
  'directory': '52cf6f28b1dbfe98f485ea78ae03942f55cd8fa0',
  'id': 'd7f30392ddda27613e0ea05cb60ec985b4f75e5c',
  'merge': True,
  'message': 'Merge pull request #4729 from simon04/provide-plugin-es2015\n\nProvidePlugin: add test case for ES2015 modules',
  'metadata': {},
  'parents': ['88f37348e7de240d794713c0c38170f17a0a8c0e',
              'd0bbf967fb51c031e16c5dfe040afce9a4113b5b'],
  'synthetic': False,
  'type': 'git'},
 {'author': {
    'email': 'kevin@fossa.io',
    'fullname': 'Kevin Wang <kevin@fossa.io>',
    'id': 14855611,
    'name': 'Kevin Wang'
  },
  'committer': {
    'email': 'noreply@github.com',
    'fullname': 'GitHub <noreply@github.com>',
    'id': 10932771,
    'name': 'GitHub'
  },
  'committer_date': '2017-04-21T18:28:41-07:00',
  'date': '2017-04-21T18:28:41-07:00',
  'directory': '412c6f5c00ce7d02efcdd6f23160a36eb4e5a1f4',
  'id': 'ff211108d888908d41470cea6187133ccdb56e87',
  'merge': False,
  'message': 'Add license scan report and status',
  'metadata': {},
  'parents': ['d7f30392ddda27613e0ea05cb60ec985b4f75e5c'],
  'synthetic': False,
  'type': 'git'},
]